import sys
import time 
from PhidgetHelperFunctions import *
from Phidget22.Devices.DigitalInput import *
from Phidget22.PhidgetException import *
from Phidget22.Phidget import *
from Phidget22.Net import *

'''
* Displays info about the attached phidget channel.
* Fired when a Phidget channel with onAttachHandler registered attaches
*
* @param self The Phidget channel that fired the attach event
'''
def onAttachHandler(self):
    
    ph = self

    try:
        serialNumber = ph.getDeviceSerialNumber()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get DeviceSerialNumber: \n\t")
        DisplayError(e)
        return

    try:
        channel = ph.getChannel()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get Channel: \n\t")
        DisplayError(e)
        return

    # Check if this is a VINT device
    try:
        hub = ph.getHub()
    except PhidgetException as e:
        if(e.code == ErrorCode.EPHIDGET_WRONGDEVICE):
            print("\nAttach Event:\n\t-> Serial Number: " + str(serialNumber) +
                    "\n\t-> Channel " + str(channel) + "\n")
        return
    try:
        hubPort = ph.getHubPort()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get HubPort: \n\t")
        DisplayError(e)
        return

    print("\nAttach Event:\n\t-> Serial Number: " + str(serialNumber) +
          "\n\t-> Hub Port: " + str(hubPort) + "\n\t-> Channel " + str(channel) + "\n")
    return

"""
* Displays info about the detached phidget channel.
* Fired when a Phidget channel with onDetachHandler registered detaches
*
* @param self The Phidget channel that fired the attach event
"""
def onDetachHandler(self):

    ph = self

    try:
        serialNumber = ph.getDeviceSerialNumber()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get DeviceSerialNumber: \n\t")
        DisplayError(e)
        return

    try:
        channel = ph.getChannel()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get Channel: \n\t")
        DisplayError(e)
        return

     # Check if this is a VINT device
    try:
        hub = ph.getHub()
    except PhidgetException as e:
        if(e.code == ErrorCode.EPHIDGET_WRONGDEVICE):
            print("\nDetach Event:\n\t-> Serial Number: " + str(serialNumber) +
                    "\n\t-> Channel " + str(channel) + "\n")
        return

    try:
        hubPort = ph.getHubPort()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get HubPort: \n\t")
        DisplayError(e)
        return

    print("\nDetach Event:\n\t-> Serial Number: " + str(serialNumber) +
          "\n\t-> Hub Port: " + str(hubPort) + "\n\t-> Channel " + str(channel) + "\n")
    return

"""
* Writes phidget error info to stderr.
* Fired when a Phidget channel with onErrorHandler registered encounters an error in the library
*
* @param self The Phidget channel that fired the attach event
* @param errorCode the code associated with the error of enum type ph.ErrorEventCode
* @param errorString string containing the description of the error fired
"""
def onErrorHandler(self, errorCode, errorString):

    sys.stderr.write("[Phidget Error Event] -> " + errorString + " (" + str(errorCode) + ")\n")

"""
* Sets the event handlers for Phidget Attach, Phidget Detach, Phidget Error events
*
* @param ph The Phidget channel to add event handlers to
* @return if the operation succeeds
* @raise PhidgetException if it fails
"""
def SetAttachDetachError_Handlers(ph):
    print("\n--------------------------------------")
    print("\nSetting OnAttachHandler...")
    try:
        ph.setOnAttachHandler(onAttachHandler)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Set Attach Handler: \n\t")
        DisplayError(e)
        raise

    print("Setting OnDetachHandler...")
    try:
        ph.setOnDetachHandler(onDetachHandler)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Set Detach Handler: \n\t")
        DisplayError(e)
        raise

    print("Setting OnErrorHandler...")
    try:
        ph.setOnErrorHandler(onErrorHandler)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Set Error Handler: \n\t")
        DisplayError(e)
        raise

    return

"""
* Outputs the DigitalInput's most recently reported state.
* Fired when a DigitalInput channel with onStateChangeHandler registered meets DataInterval and ChangeTrigger criteria
*
* @param self The DigitalInput channel that fired the StateChange event
* @param state The reported state from the DigitalInput channel
"""
def onStateChangeHandler(self, state):

    print("[State Event] -> State: " + str(state))

"""
* Creates a new instance of a DigitalInput channel.
*
* @param pdih Pointer to the PhidgetDigitalInputHandle channel to create
* @return if the operation succeeds
* @raise PhidgetException if it fails
"""
def CreateDigitalInput(pdih):

    print("Creating DigitalInput Channel...")
    try:
        pdih.create()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Creating DigitalInput: \n\t")
        DisplayError(e)
        raise

    return

"""
* Sets the event handler for DigitalInput's StateChange event
*
* @param pdih The PhidgetDigitalInputHandle channel to add the event to
* @param fptr The callback function to be called when a StateChange event is fired
* @return if the operation succeeds
* @raise PhidgetException if it fails
"""
def SetStateHandler(pdih, fptr):

    if (not (fptr is None)):
        print("\n--------------------\n"
            "\n  | State change events contain the most recent state received from the device.\n"
            "  | The linked StateChange function will run as an event every time the state changes.\n"
            "  | StateChange events will only occur when a state change happens, therefore no DataInterval is available.\n");
    
    print("")
    if (fptr is None):
        print("Clearing OnStateChangeHandler...")
    else:
        print("Setting OnStateChangeHandler...")

    print("\n--------------------")
    try:
        pdih.setOnStateChangeHandler(fptr)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Setting StateChangeHandler: \n\t")
        DisplayError(e)
        raise
    
    return

"""
* Creates, configures, and opens a DigitalInput channel.
* Displays State Change events for 10 seconds
* Closes out DigitalInput channel
*
* @return 0 if the program exits successfully, 1 if it exits with errors.
"""
def main():
    try:
        """
        * Allocate a new Phidget Channel object
        """
        try:
            ch = DigitalInput()
        except PhidgetException as e:
            sys.stderr.write("Runtime Error -> Creating DigitalInput: \n\t")
            DisplayError(e)
            raise
        except RuntimeError as e:
            sys.stderr.write("Runtime Error -> Creating DigitalInput: \n\t" + e)
            raise

        """
        * Set matching parameters to specify which channel to open
        """
        SetSerialNumber(ch)
        
        SetVINTProperties(ch)
        
        SetChannel(ch)
        
        SetupNetwork(ch)
        
        """
        * Add event handlers before calling open so that no events are missed.
        """
        SetAttachDetachError_Handlers(ch)
        
        SetStateHandler(ch, onStateChangeHandler)
        
        """
        * Open the channel with a timeout
        """
        OpenPhidgetChannel_waitForAttach(ch, 5000)

        print("Sampling data for 10 seconds...")

        time.sleep(10)

        """
         * Perform clean up and exit
         """

        SetStateHandler(ch, None)
        
        print("\nDone Sampling...")

        print("Cleaning up...")
        ClosePhidgetChannel(ch)
        print("\nExiting...")
        print("Press ENTER to end program.")
        readin = sys.stdin.readline(1)
        return 0

    except PhidgetException as e:
        sys.stderr.write("\nExiting with error(s)...")
        DisplayError(e)
        print("Press ENTER to end program.")
        readin = sys.stdin.readline(1)
        return 1

main()

